#!/usr/bin/env python
import sys
import unittest
import glob


# Run all tests
def main(pattern='test_*.py', *args):
    test_loader = unittest.TestLoader()
    test_suite = test_loader.discover('tests', pattern=pattern)
    runner = unittest.TextTestRunner()
    results = runner.run(test_suite)
    return results.wasSuccessful()

def write_duplicate_test():
    #I'm gonna automatically write the tests before running them
    #This is just a simple duplicate option to test the workflows
    script = """import unittest
import glob
from src.workflow import Workflow

class TestWorkflows(unittest.TestCase):
    """

    workflows = glob.glob(f'./tests/ressources/workflows/*', recursive=False)
    for wf in workflows:
        num = wf.split("/")[-1]
        text=f"""
        def test_wf{num}_simple_duplicate(self):
            w = Workflow(f"tests/ressources/workflows/{num}", display_info=False, duplicate=True)
            w.initialise()
            self.assertTrue(w.check_if_json_equal_to_full_structure('tests/ressources/workflows/{num}/specification_graph.json'))
        """

        script+=text
    
    with open("./tests/test_workflows_simple_duplicate.py", "w") as text_file:
        text_file.write(script)

def write_executions_extraction_test():
    #I'm gonna automatically write the tests before running them
    script = """import unittest
import json
import glob
from src.workflow import Workflow

class Test_All_Executors_Extraction(unittest.TestCase):
    """

    workflows = glob.glob(f'./tests/ressources/workflows/*', recursive=False)
    for wf in workflows:
        num = wf.split("/")[-1]
        text=f"""
        def test_{num}_extraction(self):
            w = Workflow(f"tests/ressources/workflows/{num}", display_info=False, duplicate=True)
            w.initialise()
            
            with open('tests/ressources/workflows/{num}/all_executors.json') as json_file:
                saved = json.load(json_file)

            executors = w.get_workflow_main().get_all_executors_in_workflow()
            dico= {{}}
            for e in executors:
                dico[str(e)] = e.get_code(get_OG = True)
            
            self.assertTrue(set(dico.values())==set(saved.values()))
        """

        script+=text
    
    with open("./tests/test_workflows_all_executors.py", "w") as text_file:
        text_file.write(script)

def write_process_extraction_test():
    #I'm gonna automatically write the tests before running them
    script = """import unittest
import json
import glob
from src.workflow import Workflow

class Test_Process_Extraction(unittest.TestCase):
    """

    workflows = glob.glob(f'./tests/ressources/workflows/*', recursive=False)
    for wf in workflows:
        num = wf.split("/")[-1]
        text=f"""
        def test_{num}_extraction(self):
            w = Workflow(f"tests/ressources/workflows/{num}", display_info=False, duplicate=True)
            w.initialise()
            
            with open('tests/ressources/workflows/{num}/all_processes.json') as json_file:
                saved = json.load(json_file)

            processes = w.get_processes_called()
            dico= {{}}
            for e in processes:
                dico[str(e)] = e.get_code(get_OG = True)
            
            self.assertTrue(set(dico.values())==set(saved.values()))
        """

        script+=text
    
    with open("./tests/test_process_extraction.py", "w") as text_file:
        text_file.write(script)

def write_subworkflow_extraction_test():
    #I'm gonna automatically write the tests before running them
    script = """import unittest
import json
import glob
from src.workflow import Workflow

class Test_Subworkflow_Extraction(unittest.TestCase):
    """

    workflows = glob.glob(f'./tests/ressources/workflows/*', recursive=False)
    for wf in workflows:
        num = wf.split("/")[-1]
        text=f"""
        def test_{num}_extraction(self):
            w = Workflow(f"tests/ressources/workflows/{num}", display_info=False, duplicate=True)
            w.initialise()
            
            with open('tests/ressources/workflows/{num}/all_subworkflows.json') as json_file:
                saved = json.load(json_file)

            subworkflows = w.get_subworkflows_called()
            dico= {{}}
            for e in subworkflows:
                dico[str(e)] = e.get_code(get_OG = True)
            
            self.assertTrue(set(dico.values())==set(saved.values()))
        """

        script+=text
    
    with open("./tests/test_subworkflow_extraction.py", "w") as text_file:
        text_file.write(script)

def write_executor_extraction_per_subworkflow_test():
    #I'm gonna automatically write the tests before running them
    script = """import unittest
import json
import glob
from src.workflow import Workflow

class Test_Executor_Extraction_Per_Subworkflow(unittest.TestCase):
    """

    workflows = glob.glob(f'./tests/ressources/workflows/*', recursive=False)
    for wf in workflows:
        num = wf.split("/")[-1]
        text=f"""
        def test_{num}_extraction(self):
            w = Workflow(f"tests/ressources/workflows/{num}", display_info=False, duplicate=True)
            w.initialise()
            
            with open('tests/ressources/workflows/{num}/executors_per_subworkflows.json') as json_file:
                saved = json.load(json_file)
            saved_sets = []
            for sub in saved:
                saved_sets.append(set(saved[sub].values()))
                
            subs = w.get_subworkflows_called()
            dico= {{}}
            for s in subs:
                executors = s.get_all_executors_in_workflow()
                executors_vals = []
                for e in executors:
                    executors_vals.append(e.get_code(get_OG = True))
                self.assertTrue(set(executors_vals) in saved_sets)
        """

        script+=text
    
    with open("./tests/test_executor_extraction_per_subworkflow.py", "w") as text_file:
        text_file.write(script)


if __name__ == '__main__':

    write_duplicate_test()
    write_executions_extraction_test()
    write_process_extraction_test()
    write_subworkflow_extraction_test()
    write_executor_extraction_per_subworkflow_test()
    sys.exit(0 if main(*sys.argv[1:]) else 1)
